@@ -1,22 +1,21 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2005
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2009
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
- * GNU GLOBAL is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * GNU GLOBAL is distributed in the hope that it will be useful,
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -93,14 +92,14 @@ dbop_open(const char *path, int mode, int perm, int flags)
 		info.flags |= R_DUP;
 	info.psize = DBOP_PAGESIZE;
 	/*
-	 * accept user's request but needs 0.5MB at least.
-	 * The default value is 5MB.
+	 * Decide cache size. The default value is 5MB.
+	 * See libutil/gparam.h for the details.
 	 */
-	info.cachesize = 5000000;
+	info.cachesize = GTAGSCACHE;
 	if (getenv("GTAGSCACHE") != NULL)
 		info.cachesize = atoi(getenv("GTAGSCACHE"));
-	if (info.cachesize < 500000)
-		info.cachesize = 500000;
+	if (info.cachesize < GTAGSMINCACHE)
+		info.cachesize = GTAGSMINCACHE;
 
 	/*
 	 * if unlink do job normally, those who already open tag file can use
@@ -111,7 +110,7 @@ dbop_open(const char *path, int mode, int perm, int flags)
 	db = dbopen(path, rw, 0600, DB_BTREE, &info);
 	if (!db)
 		return NULL;
-	dbop = (DBOP *)check_malloc(sizeof(DBOP));
+	dbop = (DBOP *)check_calloc(sizeof(DBOP), 1);
 	if (path == NULL)
 		dbop->dbname[0] = '\0';
 	else
@@ -311,7 +310,7 @@ dbop_first(DBOP *dbop, const char *name, regex_t *preg, int flags)
 			status == RET_SUCCESS;
 			status = (*db->seq)(db, &key, &dat, R_NEXT)) {
 			/* skip meta records */
-			if (ismeta(key.data))
+			if (ismeta(key.data) && !(dbop->openflags & DBOP_RAW))
 				continue;
 			if (preg && regexec(preg, (char *)key.data, 0, 0, 0) != 0)
 				continue;
@@ -320,6 +319,8 @@ dbop_first(DBOP *dbop, const char *name, regex_t *preg, int flags)
 	}
 	dbop->lastdat = (char *)dat.data;
 	dbop->lastsize = dat.size;
+	dbop->lastkey = (char *)key.data;
+	dbop->lastkeysize = key.size;
 	switch (status) {
 	case RET_SUCCESS:
 		break;
@@ -351,13 +352,19 @@ dbop_next(DBOP *dbop)
 	DBT key, dat;
 	int status;
 
+	if (dbop->unread) {
+		dbop->unread = 0;
+		return dbop->lastdat;
+	}
 	while ((status = (*db->seq)(db, &key, &dat, R_NEXT)) == RET_SUCCESS) {
 		assert(dat.data != NULL);
 		/* skip meta records */
-		if (flags & DBOP_KEY && ismeta(key.data))
-			continue;
-		else if (ismeta(dat.data))
-			continue;
+		if (!(dbop->openflags & DBOP_RAW)) {
+			if (flags & DBOP_KEY && ismeta(key.data))
+				continue;
+			else if (ismeta(dat.data))
+				continue;
+		}
 		if (flags & DBOP_KEY) {
 			if (!strcmp(dbop->prev, (char *)key.data))
 				continue;
@@ -367,6 +374,8 @@ dbop_next(DBOP *dbop)
 		}
 		dbop->lastdat	= (char *)dat.data;
 		dbop->lastsize	= dat.size;
+		dbop->lastkey = (char *)key.data;
+		dbop->lastkeysize = key.size;
 		if (flags & DBOP_PREFIX) {
 			if (strncmp((char *)key.data, dbop->key, dbop->keylen))
 				return NULL;
@@ -382,6 +391,18 @@ dbop_next(DBOP *dbop)
 		die("dbop_next failed.");
 	return NULL;
 }
+/*
+ * dbop_unread: unread record to read again.
+ * 
+ *	i)	dbop	dbop descripter
+ *
+ * Dbop_next will read this record later.
+ */
+void
+dbop_unread(DBOP *dbop)
+{
+	dbop->unread = 1;
+}
 /*
  * dbop_lastdat: get last data
  * 
