@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2002
+ * Copyright (c) 1997, 1998, 1999, 2000, 2002, 2008
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -70,6 +70,50 @@ setupvariables(int verbose)
 		makeobjdir = "obj";
 	}
 }
+/*
+ * getobjdir: get objdir if it exists.
+ *
+ *	i)	candidate candidate root directory
+ *	i)	verbose	verbose mode 1: on, 0: off
+ *	r)		objdir(NULL: not found)
+ */
+char *
+getobjdir(const char *candidate, int verbose)
+{
+	static char path[MAXPATHLEN+1];
+
+	/*
+	 * setup makeobjdir and makeobjdirprefix (only first time).
+	 */
+	if (makeobjdir == NULL)
+		setupvariables(0);
+	snprintf(path, sizeof(path), "%s/%s", candidate, makeobjdir);
+	if (test("d", path)) {
+		if (!test("drw", path))
+			die("Found objdir '%s', but you don't have read/write permission for it.", path);
+		if (verbose)
+			fprintf(stderr, "Using objdir '%s'.\n", path);
+		return path;
+	}
+#if !defined(_WIN32) && !defined(__DJGPP__)
+	if (test("d", makeobjdirprefix)) {
+		snprintf(path, sizeof(path), "%s%s", makeobjdirprefix, candidate);
+		if (test("d", path)) {
+			if (!test("drw", path))
+				die("Found objdir '%s', but you don't have read/write permission for it.", path);
+			if (verbose)
+				fprintf(stderr, "Using objdir '%s'.\n", path);
+			return path;
+		}
+		if (makedirectories(makeobjdirprefix, candidate + 1, verbose) < 0)
+			die("Found the base for objdir '%s', but you cannot create new directory in it.", path);
+		if (verbose)
+			fprintf(stderr, "Using objdir '%s'.\n", path);
+		return path;
+	}
+#endif
+	return NULL;
+}
 /*
  * gtagsexist: test whether GTAGS's existence.
  *
@@ -127,12 +171,16 @@ gtagsexist(const char *candidate, char *dbpath, int size, int verbose)
 #endif
 	return 0;
 }
+static char dbpath[MAXPATHLEN+1];
+static char root[MAXPATHLEN+1];
+static char root_with_slash[MAXPATHLEN+1];
+static char cwd[MAXPATHLEN+1];
 /*
  * getdbpath: get dbpath directory
  *
- *	o)	cwd	current directory
- *	o)	root	root of source tree
- *	o)	dbpath	directory which GTAGS exist
+ *	o)	a_cwd	current directory
+ *	o)	a_root	root of source tree
+ *	o)	a_dbpath directory which GTAGS exist
  *	i)	verbose	verbose mode 1: on, 0: off
  *
  * root and dbpath assumed as
@@ -147,7 +195,29 @@ gtagsexist(const char *candidate, char *dbpath, int size, int verbose)
  * and exit.
  */
 void
-getdbpath(char *cwd, char *root, char *dbpath, int verbose)
+getdbpath(char *a_cwd, char *a_root, char *a_dbpath, int verbose)
+{
+	/*
+	 * get the values.
+	 */
+	setupdbpath(verbose);
+	/*
+	 * return the values.
+	 */
+	strlimcpy(a_dbpath, dbpath, MAXPATHLEN);
+	strlimcpy(a_root, root, MAXPATHLEN);
+	strlimcpy(a_cwd, cwd, MAXPATHLEN);
+}
+/*
+ * setupdbpath: setup dbpath directory
+ *
+ *	go)	cwd	current directory
+ *	go)	root	root of source tree
+ *	go)	dbpath directory which GTAGS exist
+ *	i)	verbose	verbose mode 1: on, 0: off
+ */
+void
+setupdbpath(int verbose)
 {
 	struct stat sb;
 	char *p;
@@ -235,4 +305,31 @@ getdbpath(char *cwd, char *root, char *dbpath, int verbose)
 			break;
 		} while (0);
 	}
+	if (!strcmp(root, "/"))
+		strlimcpy(root_with_slash, root, sizeof(root_with_slash));
+	else
+		snprintf(root_with_slash, sizeof(root_with_slash), "%s/", root);
+}
+/*
+ * return saved values.
+ */
+const char *
+get_dbpath()
+{
+	return (const char *)dbpath;
+}
+const char *
+get_root()
+{
+	return (const char *)root;
+}
+const char *
+get_root_with_slash()
+{
+	return (const char *)root_with_slash;
+}
+const char *
+get_cwd()
+{
+	return (const char *)cwd;
 }
